﻿using System;
using NUnit.Framework;
using Telerik.OpenAccess;
using FluentAPI.Infastructure;
using FluentAPI.Logger;
using FluentAPI.Model;

namespace FluentAPI.Tests
{
    /// <summary>
    /// Using Nunit, execute tests that demonstrate Forward mapping via OpenAccess FluentAPI
    /// The following line must be added to your .csproj file for OpenAccess to work properly
    /// <Import Condition="Exists('$(MSBuildExtensionsPath)\OpenAccess.targets')" Project="$(MSBuildExtensionsPath)\OpenAccess.targets" />
    /// By Ed Charbeneau (twitter:@EdCharbeneau)
    /// </summary>
    [TestFixture]
    [Category("Integration")]
    public class Forward_Mapping_Tests
    {
        /// <summary>
        /// This demo connection string uses Windows Authentication
        /// The user should have the necessary privlages to create a database.
        /// "Data Source=.\SQLEXPRESS;Initial Catalog={database};Integrated Security=True;Connect Timeout=2;User Instance=False"
        /// </summary>
        private string connectionString = @"Data Source=.\SQLEXPRESS;Initial Catalog={0};Integrated Security=True;Connect Timeout=2;User Instance=False";
        private string databaseName = "FluentAPI_Tests";
        private IContextFactory context;
        private ILogger logger;

        [TestFixtureSetUp]
        public void TextFixtureSetup()
        {
            connectionString = string.Format(connectionString, databaseName);
            context = new ContextFactory(connectionString);
            logger = new ConsoleLogger();
        }

        [Test]
        public void Create_A_Test_Database()
        {
            ISchemaHandler schema;
            string ddlScript;

            schema = context.Get().GetSchemaHandler();

            //Drop database
            //Uncomment if you would like the test project to drop any existing database with the name specified in 'databaseName' (may not work with all database engines)
            /*
            try
            {
                logger.Log("Deleting test database");
                ddlScript = "USE Master ALTER DATABASE {0} SET SINGLE_USER WITH ROLLBACK IMMEDIATE DROP DATABASE {0}";
                ddlScript = string.Format(ddlScript, databaseName);
                schema.ExecuteDDLScript(ddlScript);
                ddlScript = "";
                logger.Log("Test database deleted");
            }
            catch (Exception ex)
            {
                logger.Log("Deleting test database failed with: {0}", ex.Message);
            }
             */

            if (!schema.DatabaseExists())
            {
                //Database does not exist
                //Use OpenAccess to generate a Create Database SQL script
                schema.CreateDatabase();
                ddlScript = schema.CreateDDLScript();
            }
            else
            {
                //Database exists
                //Use OpenAccess to generate a Update Database SQL script
                ddlScript = schema.CreateUpdateDDLScript(null);
            }

            //Execute ddlScript if a script and connection exist
            if (!string.IsNullOrEmpty(ddlScript) && !string.IsNullOrEmpty(connectionString))
            {
                {
                    try
                    {
                        logger.Log("Creating test database");
                        schema.ExecuteDDLScript(ddlScript);
                        logger.Log("Test database created");
                    }
                    catch (Exception ex)
                    {
                        logger.Log("Creating test database failed with: {0}", ex.Message);
                    }
                }
            }
            else
            {
                logger.Log("No DDL script Executed.");
                logger.Log("Connection string was: {0}", connectionString);
                logger.Log("DDL script was: {0}", ddlScript);
            }
        }

        [Test]
        public void Should_Create_Company()
        {

        	//Arrange
            OpenAccessContext oaContext = context.Get();
            string companyName = "Alpha Company";

            //Act
            Company CompanyForInsert = new Company() { Name = companyName };
            //Note: Directly accessing the OpenAccess contexts Add and SaveChanges methods is not generally the best practice for most projects.
            //      Instead a service layer should exist here.
            oaContext.Add(CompanyForInsert);
            oaContext.SaveChanges();
            //Assert
            Assert.IsTrue(CompanyForInsert.Id > 0, "An id was not returned");
        }

        [Test]
        public void Should_Create_Location()
        {
            //Arrange
            OpenAccessContext oaContext = context.Get();
            string companyName = "Beta Company";
            string locationName = "Area 1";

            //Act

            Company CompanyForInsert = new Company() { Name = companyName };
            Location LocationForInsert = new Location() { Name = locationName };

            oaContext.Add(CompanyForInsert);
            oaContext.SaveChanges();

            //Note: Directly accessing the OpenAccess contexts Add and SaveChanges methods is not generally the best practice for most projects.
            //      Instead a service layer should exist here.
            //assign parent entity
            LocationForInsert.Company = CompanyForInsert;
            //add to child to parent
            CompanyForInsert.Locations.Add(LocationForInsert);
            oaContext.SaveChanges();

            //Assert
            Assert.IsTrue(locationName == LocationForInsert.Name, "The expected Location Name does not match");
            Assert.IsTrue(LocationForInsert.Id > 0, "An id was not returned");
            Assert.IsTrue(CompanyForInsert.Locations.Count == 1);
        }

        [Test]
        public void Should_Create_MsdsItem()
        {
            //Arange
            OpenAccessContext oaContext = context.Get();
            string companyName = "Espilon Company";
            string locationName = "Dorm 1";

            MsdsItem MsdsItemForInsert = new MsdsItem()
            {
                CreatedBy = "@EdCharbeneau",
                FileName = "Fake.pdf",
                FireRisk = HazardLevel.Minimal,
                HealthRisk = HazardLevel.Moderate,
                IsArchived = false,
                ManufacturerAddress = "2044 Recursive loop",
                ManufacturerName = "A1 Telerik Manufacturing",
                Name = "JustClean",
                PartNumber = "9999",
                ProductState = "Solid",
                ReactivityHazard = HazardLevel.Severe,
                SpecialHazard = SpecialHazard.Corrosive,
                Usage = "Cleaner",
                Warning = "Should not come in contact with skin."
            };
            //Act
            Company CompanyForInsert = new Company() { Name = companyName };
            Location LocationForInsert = new Location() { Name = locationName };

            //Note: Directly accessing the OpenAccess contexts Add and SaveChanges methods is not generally the best practice for most projects.
            //      Instead a service layer should exist here.
            oaContext.Add(CompanyForInsert);
            oaContext.SaveChanges();

            LocationForInsert.Company = CompanyForInsert;
            CompanyForInsert.Locations.Add(LocationForInsert);
            oaContext.SaveChanges();
            //add simulated Msds item
            MsdsItemForInsert.Location = LocationForInsert;
            LocationForInsert.MsdsItems.Add(MsdsItemForInsert);
            oaContext.SaveChanges();

            //Assert
            Assert.IsTrue(MsdsItemForInsert.Id > 0, "An id was not returned");
            Assert.IsTrue(LocationForInsert.MsdsItems.Count == 1);
        }

    }


}
